package ru.landgrafhomyak.db.raw_sql_skeleton

public class Table(
	public val namespace: Namespace,
	public val name: String
) {
	public val asSqlReference: String = "\"${this.name}\""
	public val asSqlString: String = "\'${this.name}\'"

	override fun toString(): String = this.asSqlReference

	public interface Constructor<TableUserExtension : Any> {
		public fun createTable(table: Table, members: Scope): TableUserExtension

		public class Scope internal constructor(private val _table: Table) {
			private val _usedNames = HashSet<String>()

			private fun _allocName(name: String): String {
				if (!this._usedNames.add(name))
					throw IllegalArgumentException("Name duplication: $name")
				return name
			}

			public fun column(name: String, type: String): Column =
				Column(this._table, this._allocName(name), type)

			public fun column(name: String, referenceType: String, declarationType: String): Column =
				Column(this._table, this._allocName(name), referenceType, declarationType)

			public fun index(name: String): NamedTableMember =
				NamedTableMember(this._table, this._allocName(name))

			public fun uniqueIndex(name: String): NamedTableMember =
				NamedTableMember(this._table, this._allocName(name))

			public fun checkConstraint(name: String): NamedTableMember =
				NamedTableMember(this._table, this._allocName(name))
		}
	}
}